@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (c) 2013-2014 Google, Inc.
 # Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
@@ -25,6 +24,8 @@
 # Copyright (c) 2019 Grygorii Iermolenko <gyermolenko@gmail.com>
 # Copyright (c) 2020 David Gilman <davidgilman1@gmail.com>
 # Copyright (c) 2020 Peter Kolbus <peter.kolbus@gmail.com>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 # Copyright (c) 2021 Damien Baty <damien@damienbaty.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
@@ -98,6 +99,11 @@ import astroid
 import astroid.test_utils as test_utils
 
 
+def assertEqualMro(klass, expected_mro):
+    """Check mro names."""
+    assert [member.name for member in klass.mro()] == expected_mro
+
+
 class HashlibTest(unittest.TestCase):
     def _assert_hashlib_class(self, class_obj):
         self.assertIn("update", class_obj)
@@ -569,10 +575,10 @@ class MultiprocessingBrainTest(unittest.TestCase):
         """
         )
         ast_queue = next(module["queue"].infer())
-        self.assertEqual(ast_queue.qname(), "{}.Queue".format(queue.__name__))
+        self.assertEqual(ast_queue.qname(), f"{queue.__name__}.Queue")
 
         joinable_queue = next(module["joinable_queue"].infer())
-        self.assertEqual(joinable_queue.qname(), "{}.Queue".format(queue.__name__))
+        self.assertEqual(joinable_queue.qname(), f"{queue.__name__}.Queue")
 
         event = next(module["event"].infer())
         event_name = "threading.Event"
@@ -592,7 +598,7 @@ class MultiprocessingBrainTest(unittest.TestCase):
 
         for attr in ("list", "dict"):
             obj = next(module[attr].infer())
-            self.assertEqual(obj.qname(), "{}.{}".format(bases.BUILTINS, attr))
+            self.assertEqual(obj.qname(), f"{bases.BUILTINS}.{attr}")
 
         # pypy's implementation of array.__spec__ return None. This causes problems for this inference.
         if not hasattr(sys, "pypy_version_info"):
@@ -633,12 +639,10 @@ class ThreadingBrainTest(unittest.TestCase):
 
     def _test_lock_object(self, object_name):
         lock_instance = builder.extract_node(
-            """
+            f"""
         import threading
-        threading.{}()
-        """.format(
-                object_name
-            )
+        threading.{object_name}()
+        """
         )
         inferred = next(lock_instance.infer())
         self.assert_is_valid_lock(inferred)
@@ -676,7 +680,7 @@ class EnumBrainTest(unittest.TestCase):
         one = enumeration["one"]
         self.assertEqual(one.pytype(), ".MyEnum.one")
 
-        property_type = "{}.property".format(bases.BUILTINS)
+        property_type = f"{bases.BUILTINS}.property"
         for propname in ("name", "value"):
             prop = next(iter(one.getattr(propname)))
             self.assertIn(property_type, prop.decoratornames())
@@ -748,7 +752,7 @@ class EnumBrainTest(unittest.TestCase):
         one = enumeration["one"]
 
         clazz = one.getattr("__class__")[0]
-        int_type = "{}.{}".format(bases.BUILTINS, "int")
+        int_type = f"{bases.BUILTINS}.int"
         self.assertTrue(
             clazz.is_subtype_of(int_type),
             "IntEnum based enums should be a subtype of int",
@@ -973,12 +977,10 @@ class IOBrainTest(unittest.TestCase):
     def test_sys_streams(self):
         for name in {"stdout", "stderr", "stdin"}:
             node = astroid.extract_node(
-                """
+                f"""
             import sys
-            sys.{}
-            """.format(
-                    name
-                )
+            sys.{name}
+            """
             )
             inferred = next(node.infer())
             buffer_attr = next(inferred.igetattr("buffer"))
@@ -1193,6 +1195,132 @@ class TypingBrain(unittest.TestCase):
         inferred = next(node.infer())
         self.assertIsInstance(inferred, astroid.Instance)
 
+    @test_utils.require_version("3.8")
+    def test_typedDict(self):
+        node = builder.extract_node(
+            """
+        from typing import TypedDict
+        class CustomTD(TypedDict):
+            var: int
+        """
+        )
+        assert len(node.bases) == 1
+        inferred_base = next(node.bases[0].infer())
+        self.assertIsInstance(inferred_base, nodes.ClassDef, node.as_string())
+        typing_module = inferred_base.root()
+        assert len(typing_module.locals["TypedDict"]) == 1
+        assert inferred_base == typing_module.locals["TypedDict"][0]
+
+    @test_utils.require_version("3.8")
+    def test_typing_alias_type(self):
+        """
+        Test that the type aliased thanks to typing._alias function are
+        correctly inferred.
+        """
+
+        def check_metaclass(node: nodes.ClassDef):
+            meta = node.metaclass()
+            assert isinstance(meta, nodes.ClassDef)
+            assert meta.name == "ABCMeta_typing"
+            assert "ABCMeta" == meta.basenames[0]
+            assert meta.locals.get("__getitem__") is not None
+
+            abc_meta = next(meta.bases[0].infer())
+            assert isinstance(abc_meta, nodes.ClassDef)
+            assert abc_meta.name == "ABCMeta"
+            assert abc_meta.locals.get("__getitem__") is None
+
+        node = builder.extract_node(
+            """
+        from typing import TypeVar, MutableSet
+
+        T = TypeVar("T")
+        MutableSet[T]
+
+        class Derived1(MutableSet[T]):
+            pass
+        """
+        )
+        inferred = next(node.infer())
+        check_metaclass(inferred)
+        assertEqualMro(
+            inferred,
+            [
+                "Derived1",
+                "MutableSet_typing",
+                "MutableSet",
+                "Set",
+                "Collection",
+                "Sized",
+                "Iterable",
+                "Container",
+                "object",
+            ],
+        )
+
+        node = builder.extract_node(
+            """
+        import typing
+        class Derived2(typing.OrderedDict[int, str]):
+            pass
+        """
+        )
+        inferred = next(node.infer())
+        check_metaclass(inferred)
+        assertEqualMro(
+            inferred,
+            [
+                "Derived2",
+                "OrderedDict_typing",
+                "OrderedDict",
+                "dict",
+                "object",
+            ],
+        )
+
+        node = builder.extract_node(
+            """
+        import typing
+        class Derived3(typing.Pattern[str]):
+            pass
+        """
+        )
+        inferred = next(node.infer())
+        check_metaclass(inferred)
+        assertEqualMro(
+            inferred,
+            [
+                "Derived3",
+                "Pattern",
+                "object",
+            ],
+        )
+
+    @test_utils.require_version("3.8")
+    def test_typing_alias_side_effects(self):
+        """Test that typing._alias changes doesn't have unwanted consequences."""
+        node = builder.extract_node(
+            """
+        import typing
+        import collections.abc
+
+        class Derived(collections.abc.Iterator[int]):
+            pass
+        """
+        )
+        inferred = next(node.infer())
+        assert inferred.metaclass() is None  # Should this be ABCMeta?
+        assertEqualMro(
+            inferred,
+            [
+                "Derived",
+                # Should this be more?
+                # "Iterator_typing"?
+                # "Iterator",
+                # "object",
+            ],
+        )
+
 
 class ReBrainTest(unittest.TestCase):
     def test_regex_flags(self):
@@ -2097,6 +2225,8 @@ def test_dataclasses():
     code = """
     import dataclasses
     from dataclasses import dataclass
+    import typing
+    from typing import ClassVar
 
     @dataclass
     class InventoryItem:
@@ -2106,6 +2236,8 @@ def test_dataclasses():
     @dataclasses.dataclass
     class Other:
         name: str
+        CONST_1: ClassVar[int] = 42
+        CONST_2: typing.ClassVar[int] = 42
     """
 
     module = astroid.parse(code)
@@ -2124,6 +2256,14 @@ def test_dataclasses():
     assert len(name) == 1
     assert isinstance(name[0], astroid.Unknown)
 
+    const_1 = second.getattr("CONST_1")
+    assert len(const_1) == 1
+    assert isinstance(const_1[0], astroid.AssignName)
+
+    const_2 = second.getattr("CONST_2")
+    assert len(const_2) == 1
+    assert isinstance(const_2[0], astroid.AssignName)
+
 
 @pytest.mark.parametrize(
     "code,expected_class,expected_value",
