@@ -16,6 +16,9 @@
 # Copyright (c) 2019 Alex Hall <alex.mojaki@gmail.com>
 # Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
 # Copyright (c) 2020 David Gilman <davidgilman1@gmail.com>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
+# Copyright (c) 2021 hippo91 <guillaume.peillex@gmail.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
 # For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
@@ -115,17 +118,15 @@ class AsStringTest(resources.SysPathSetup, unittest.TestCase):
         self.assertEqual(ast.as_string(), "raise_string(*args, **kwargs)")
 
     def test_module_as_string(self):
-        """check as_string on a whole module prepared to be returned identically
-        """
+        """check as_string on a whole module prepared to be returned identically"""
         module = resources.build_file("data/module.py", "data.module")
-        with open(resources.find("data/module.py"), "r") as fobj:
+        with open(resources.find("data/module.py")) as fobj:
             self.assertMultiLineEqual(module.as_string(), fobj.read())
 
     def test_module2_as_string(self):
-        """check as_string on a whole module prepared to be returned identically
-        """
+        """check as_string on a whole module prepared to be returned identically"""
         module2 = resources.build_file("data/module2.py", "data.module2")
-        with open(resources.find("data/module2.py"), "r") as fobj:
+        with open(resources.find("data/module2.py")) as fobj:
             self.assertMultiLineEqual(module2.as_string(), fobj.read())
 
     def test_as_string(self):
@@ -402,7 +403,7 @@ class TryExceptFinallyNodeTest(_NodeTest):
 
 class ImportNodeTest(resources.SysPathSetup, unittest.TestCase):
     def setUp(self):
-        super(ImportNodeTest, self).setUp()
+        super().setUp()
         self.module = resources.build_file("data/module.py", "data.module")
         self.module2 = resources.build_file("data/module2.py", "data.module2")
 
