@@ -2,6 +2,8 @@
 # Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
 # Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
 # Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
+# Copyright (c) 2020 David Gilman <davidgilman1@gmail.com>
+# Copyright (c) 2021 hippo91 <guillaume.peillex@gmail.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
 # For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
@@ -14,7 +16,6 @@ from astroid import builder
 from astroid import exceptions
 from astroid import nodes
 from astroid import objects
-from astroid import test_utils
 
 
 class ObjectsTest(unittest.TestCase):
@@ -97,7 +98,6 @@ class SuperTests(unittest.TestCase):
         self.assertIsInstance(second, bases.Instance)
         self.assertEqual(second.qname(), "%s.super" % bases.BUILTINS)
 
-    @test_utils.require_version(minver="3.0")
     def test_no_arguments_super(self):
         ast_nodes = builder.extract_node(
             """
